import asyncio
import importlib

import carb.settings

import omni.ext

from omni.services.core import main


class KafkaConsumerExtension(omni.ext.IExt):
    """ Kafka Consumers.

        Allows Consumers to be hooked up to Kafka topics to retrieve message that will be translated and sent to services
        Multiple consumers can run at the same time handling different topics or different service calls.
    """

    def __init__(self):
        self._futures = None

    def on_startup(self, ext_id):
        self._futures = []
        settings = carb.settings.get_settings()

        ext_name = ext_id.split("-")[0]

        consumer_cls_path = settings.get(f"exts/{ext_name}/consumer_class")
        consumer_cls = self._import(consumer_cls_path)

        consumers = settings.get(f"exts/{ext_name}/consumers") or []
        for consumer_args in consumers:
            consumer = consumer_cls(
                consumer_args["bootstrap_servers"],
                consumer_args["topics"],
                consumer_args["service_calls"],
                consumer_args.get("group_id", None),
                main.get_app()
            )
            self._futures.append(asyncio.ensure_future(consumer.run()))

    def on_shutdown(self):
        for future in self._futures:
            future.cancel()

    def _import(self, import_path):
        cls_name = import_path.split(".")[-1]
        import_path = import_path.replace(f".{cls_name}", "")
        module = importlib.import_module(import_path)
        return getattr(module, cls_name)
