import json
from typing import List
import unittest

import pydantic

import omni.kit.test
from omni.services.core import main
from omni.services.transport.server.kafka import consumer


class _KafkaModel(pydantic.BaseModel):
    topic: str = pydantic.Field("", title="Kafka Topic", description="The Kafka topic the request was consumed from")
    partition: int = pydantic.Field(0, title="Kafka partition")
    offset: int = pydantic.Field(0, title="Offset")
    key: str = pydantic.Field("", title="Request Key")
    value: dict = pydantic.Field(..., title="Kafka value payload", description="Kafka value, holds the data")
    timestamp: str = pydantic.Field("", title="Timestamp")


class _KafKaDummyMessage(object):

    def __init__(self, topic: str, partition: int, offset: int, key: str, value: dict, timestamp: str) -> None:
        self.topic = topic
        self.partition = partition
        self.offset = offset
        self.key = key
        self.value = value
        self.timestamp = timestamp


class TestKafkaConsumer(omni.kit.test.AsyncTestCaseFailOnLogError):

    async def setUp(self):
        self._results: List[_KafkaModel] = []

        def check_output(data: _KafkaModel):
            self._results.append(data)

        main.register_endpoint("post", "/output", check_output)

        self._service_endpoint = "/output"

        self._consumer = consumer.KafkaConsumer(
            servers=[],
            topics=[],
            services=[self._service_endpoint],
            group_id="",
            app=main.get_app(),
        )

    @unittest.skip("Revisit this test to ensure format of the `value` and `key` properties matches the expected format.")
    async def test_process_message(self):
        # TODO: Validate if the type of the `key` and `value` properties accurately match the expected formats. Current
        # typings for `value` indicate an expected `dict`
        message = _KafKaDummyMessage(
            topic="my-test-topic",
            partition=0,
            offset=0,
            key="my-key".encode("ascii"),
            value=json.dumps({"foo": "bar"}),
            timestamp="",
        )

        await self._consumer._process_message(message, self._service_endpoint)

        self.assertGreater(len(self._results), 0)
        self.assertEqual(message, self._results[0])
